import os

from red_dead import base_dir
from red_dead.export import write_export

# private sub-module for portable local dev
AUTH_PATH = base_dir / 'red-dead-tools-private/.data/red-dead-tools-b475e08cb189.json'

EXPORT_PATH = base_dir / 'exports'


def export():
    os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = str(AUTH_PATH)

    write_export(EXPORT_PATH)


if __name__ == '__main__':
    export()



import os
import shutil
import subprocess

from red_dead.items import path_originals, base_dir


submod_path = base_dir / 'RDR2CollectorsMap'
os.chdir(submod_path)
subprocess.check_call(['git', 'pull', 'origin', 'master'])

for cached_path, submod_path in path_originals.items():
    shutil.copy(submod_path, cached_path)



from collections import defaultdict
from datetime import datetime

from .items import get_item, parse_data, get_collection
from .models import Settings
from .sheet_transform import get_col_item_needs, get_no_hide_collections


def get_important():
    imp_items = defaultdict(set)
    for col_name, items in get_col_item_needs().items():
        for item_name in items:
            of_import = not item_name.startswith('!')
            item_name = item_name.strip('!')
            item = get_item(item_name, col_name)
            print(f'{col_name:>30} - {item_name:>30} --> {item.name}')
            imp_items[of_import].add(item)

    return imp_items[True], imp_items[False]


def remove_no_hide_cols(unimportant):
    no_hide_col_names = get_no_hide_collections()
    no_hide_cols = [get_collection(name) for name in no_hide_col_names]

    return {it for it in unimportant if it.collection not in no_hide_cols}


def get_settings():
    _, items = parse_data()

    important, unimportant = get_important()

    unimportant |= (items - important)

    unimportant = remove_no_hide_cols(unimportant)

    return Settings(important_items=important, unimportant_items=unimportant)


def write_export(export_path):
    now = datetime.now()
    path = export_path / now.strftime('%Y-%m-%d_%H-%M-%S.json')

    s = get_settings()
    path.write_text(s.as_json())

    print(f'export written to {path}')



import json
from datetime import datetime, timedelta

import attr


SETTINGS = {
    "main.isMenuOpened": True,
    "main.isFmeDisplayEnabled": False,
    "main.markerColor": 'by_category',
    "main.baseLayer": 'map.layers.detailed',
    "enabled-categories": [
        'flower',
        'bottle',
        'arrowhead',
        'egg',
        'coin',
        'heirlooms',
        'bracelet',
        'earring',
        'necklace',
        'ring',
        'cups',
        'pentacles',
        'swords',
        'wands',
        'nazar',
        'fast_travel',
        'treasure',
        'user_pins',
        'random',
    ],
}


@attr.s
class Settings:
    version = 2

    static = attr.ib(default=SETTINGS)
    important_items = attr.ib(factory=list)
    unimportant_items = attr.ib(factory=list)

    def as_export(self):
        data = self.static.copy()
        for key, value in data.items():
            data[key] = json.dumps(value)

        items = [item.code for item in self.important_items]
        data['importantItems'] = json.dumps(items)

        for item in self.unimportant_items:
            data[f'amount.{item.code}'] = '1'
            data[f'collected.{item.code}'] = 'true'
            for i in range(10):
                data[f'collected.{item.code}_{i}'] = 'true'

        date = datetime.now() - timedelta(hours=2)
        data['main.date'] = date.strftime('%Y-%m-%d')
        data['length'] = len(data)
        data['version'] = self.version

        return data

    def as_json(self):
        return json.dumps(self.as_export(), indent=4)


@attr.s(frozen=True)
class Item:
    collection = attr.ib()
    code = attr.ib()
    name = attr.ib()


@attr.s(frozen=True)
class Collection:
    code = attr.ib()
    name = attr.ib()



import json
from difflib import get_close_matches
from functools import lru_cache

from . import base_dir
from .models import Collection, Item


item_path = base_dir / 'data' / 'en.json'
weekly_path = base_dir / 'data' / 'weekly.json'

path_originals = {
    item_path: base_dir / 'RDR2CollectorsMap/langs/en.json',
    weekly_path: base_dir / 'RDR2CollectorsMap/data/weekly.json',
}


@lru_cache(maxsize=None)
def parse_data():
    with item_path.open() as stream:
        data = json.load(stream)

    collections = {}
    items = {}

    for long_code, name in data.items():
        col_item_code, _, attrib = long_code.partition('.')

        if attrib != 'name':
            continue

        col_code = col_item_code.split('_', 1)[0]

        try:
            col_name = data[f'menu.{col_code}']
        except KeyError:
            continue

        try:
            col = collections[col_name]
        except KeyError:
            col = Collection(code=col_code, name=col_name)
            collections[col_name] = col

        item = Item(code=col_item_code, name=name, collection=col)
        items[col_item_code] = item

    with weekly_path.open() as stream:
        data = json.load(stream)

    current = data['current']
    weekly_item_codes = data['sets'][current]
    col = Collection(code='weekly', name='Weekly')
    for item_code in weekly_item_codes:
        orig = items[item_code]
        item = Item(code=item_code, name=orig.name, collection=col)
        items[f'weekly_{item_code}'] = item
    collections['weekly'] = col

    return set(collections.values()), set(items.values())


def numerise(text):
    num_names = 'ace two three four five six seven eight nine ten'.title().split()
    num_map = {str(i): f'{name} of ' for i, name in enumerate(num_names, 1)}
    num_map.update({'t': 'knight', 'p': 'page', 'q': 'queen', 'k': 'king'})
    return num_map.get(text.lower().strip(), text)


def guess_by_name(name, objs):
    name = numerise(name)

    obj_map = {obj.name.lower(): obj for obj in objs}
    guesses = get_close_matches(name.lower(), obj_map, n=1, cutoff=0.1)
    try:
        guess = guesses[0]
    except IndexError:
        raise ValueError(f'{name} not found in {", ".join(obj_map.keys())}')
    return obj_map[guess]


def get_collection(collection_name):
    collections, _ = parse_data()
    return guess_by_name(collection_name, collections)


def get_item(item_name, collection_name=None):
    collections, items = parse_data()

    if collection_name:
        collection = get_collection(collection_name)
        items = [i for i in items if i.collection == collection]

    return guess_by_name(item_name, items)



"""
Generic helpers for fetching data from Google Spreadsheets.
"""
import gspread
import google.auth


def gsheets_api_connect():
    creds, project_id = google.auth.default(scopes=gspread.auth.DEFAULT_SCOPES)

    return gspread.authorize(creds)


def get_sheet_rows(spreadsheet_name, sheet_name):
    conn = gsheets_api_connect()
    spreadsheet = conn.open(spreadsheet_name)
    sheet = spreadsheet.worksheet(sheet_name)
    rows = sheet.get_all_values()
    return rows



from setuptools import setup, find_packages


setup(
    name='red-dead-tools',
    version="0.1.0",
    packages=find_packages(),
    url="https://github.com/red-dead-tools/red-dead-tools",
    description="Tools for playing Red Dead Redemption 2 online",
    author="Tom Viner",
    author_email="red-dead-tools@viner.tv",
    install_requires=[
        'gspread',
        'attrs',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



"""
Specific helpers to work with my RDR2 collecting spreadsheet data.
"""
from functools import lru_cache

from .gsheets import get_sheet_rows


ITEM_ROWS = 9


@lru_cache(maxsize=None)
def get_rows(spreadsheet_name='RDR2 Collecting Needs', sheet_name="Tom"):
    return get_sheet_rows(spreadsheet_name, sheet_name)


def get_col_item_needs():
    rows = get_rows()

    data = {}
    for prefix, col_name, *items in zip(*rows[: 2 + ITEM_ROWS]):
        if prefix and col_name != 'Weekly':
            col_name = f'{prefix} - {col_name}'
        data[col_name] = [it for it in items if it]

    return data


def get_no_hide_collections():
    rows = get_rows()

    cols = set()
    no_hide_rows = [row for row in rows if row[0] == 'No hide']
    rows = rows[:2] + no_hide_rows
    for prefix, col_name, no_hide in zip(*rows):
        if prefix and col_name != 'Weekly':
            col_name = f'{prefix} - {col_name}'
        if no_hide:
            cols.add(col_name)

    return cols


if __name__ == '__main__':
    rows = get_rows()
    print(rows)
    print()

    col_item_needs = get_col_item_needs()
    print(col_item_needs)
